from textual.widget import Widget
from textual.reactive import reactive
from textual.message import Message
from textual._types import MessageTarget
from textual import events

from rich.console import RenderableType

import os
import re

from ._icons import get_icon


class FileDrop(Widget, can_focus=True, can_focus_children=False):

    DEFAULT_CSS = """
    
    FileDrop {
        border: round gray;
        height: 100%;
        background: $panel;
        content-align: center middle;
        padding: 0 3;
    }
    """

    txt = reactive("Please Drag and Drop the files here...")

    def __init__(
        self,
        display: bool = True,
        name: str | None = None,
        id: str | None = None,
        classes: str | None = None,
    ) -> None:
        super().__init__(name=name, id=id, classes=classes)

        if not display:
            self.styles.display = "none"

    def render(self) -> RenderableType:
        return self.txt

    class Dropped(Message):
        def __init__(
            self,
            sender: MessageTarget,
            path: str,
            filepaths: list,
            filenames: list,
            filesobj: list,
        ) -> None:
            self.path = path
            self.filepaths = filepaths
            self.filenames = filenames
            self.filesobj = filesobj
            super().__init__(sender)

    async def on_event(self, event: events.Event) -> None:
        if isinstance(event, events.Focus):
            self.styles.border = ("round", "dodgerblue")
        if isinstance(event, events.Blur):
            self.styles.border = ("round", "gray")
        if isinstance(event, events.Paste):
            pattern = r'(?:[^\s"]|"(?:\\"|[^"])*")+'
            split_filepaths = re.findall(pattern, event.text)
            filepaths = [
                i.replace("\x00", "").replace('"', "")
                for i in split_filepaths
                if os.path.isfile(i.replace("\x00", "").replace('"', ""))
            ]
            if filepaths:
                filenames = [os.path.basename(i) for i in filepaths]
                filesobj = []
                for i in filepaths:
                    file_name = os.path.basename(i)
                    _, file_ext = os.path.splitext(file_name)
                    file_ext = file_ext.replace(".", "")
                    file_path = i
                    filesobj.append(
                        {
                            "path": file_path,
                            "name": file_name,
                            "ext": file_ext,
                            "icon": get_icon(file_name, file_ext),
                        }
                    )
                await self.emit(
                    self.Dropped(
                        self,
                        os.path.split(filepaths[0])[0],
                        filepaths,
                        filenames,
                        filesobj,
                    )
                )
                self.txt = " ".join(
                    [
                        f'[on dodger_blue3] {i["icon"]} [/][on gray27]{i["name"]}[/]'
                        for i in filesobj
                    ]
                )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['textual_filedrop']

package_data = \
{'': ['*']}

install_requires = \
['rich>=13.0.0,<14.0.0', 'textual>=0.9.0,<0.10.0']

setup_kwargs = {
    'name': 'textual-filedrop',
    'version': '0.2.2',
    'description': 'FileDrop widget for Textual, easily drag and drop files into your terminal apps.',
    'long_description': '![textual-filedrop](https://user-images.githubusercontent.com/16024979/208708722-e550d8ca-22a7-47f0-adf9-16cad570cdfd.png)\n\n# textual-filedrop\n\nAdd filedrop support to your [Textual](https://github.com/textualize/textual/) apps, easily drag and drop files into your terminal apps.\n\n> _Tested on `Windows` and [`macOS`](https://github.com/Textualize/textual/discussions/1414#discussioncomment-4467029)._\n\n> _[Nerd Font](https://www.nerdfonts.com/font-downloads) is required to display file icons._\n\n## Install\n\n```\npip install textual-filedrop\n```\n\nor\n\n```\ngit clone https://github.com/agmmnn/textual-filedrop.git\ncd textual-filedrop\npoetry install\n```\n\n## Usage\n\nYou can find more examples [here](./examples).\n\n```py\nfrom textual_filedrop import FileDrop\n```\n\n```py\n# add FileDrop widget to your app\nyield FileDrop(id="filedrop")\n```\n\n```py\n# focus the widget\nself.query_one("#filedrop").focus()\n```\n\n```py\n# when the files are dropped\ndef on_file_drop_dropped(self, message: FileDrop.Dropped) -> None:\n    path = message.path\n    filepaths = message.filepaths\n    filenames = message.filenames\n    filesobj = message.filesobj\n    print(path, filepaths, filenames, filesobj)\n\n\n# output: path, [filepaths], [filenames], [filesobj]\n```\n\n## Examples\n\n### [subdomain_lister](./examples/subdomain_lister.py)\n\nDrag and drop the subdomain list files and see the results as a tree list.\n\n![subdomain_lister](https://user-images.githubusercontent.com/16024979/208706132-0a33bb21-51b8-441a-aeb9-668dbfcb382c.gif)\n\n### [fullscreen](./examples/fullscreen.py)\n\nFullscreen example, will show the results in the textual console.\n\n### [hidden](./examples/hidden.py)\n\nAs long as focus is on, the FileDrop widget will be active even if it is not visible on the screen.\n\n## Dev\n\n```\npoetry install\n\ntextual console\npoetry run textual run --dev examples/subdomain_lister.py\n```\n',
    'author': 'agmmnn',
    'author_email': 'agmmnn@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.8,<4.0.0',
}


setup(**setup_kwargs)
